// Air Conditioning Control With Blynk App
// 2020 Gord Payne   www.hausOfPayne.weebly.com/arduino
//                   youtube link here
// Turn AC on/off via Blynk App or IR remote
// Night Mode with timed On/Off cycles
// Temperature and Humidity Reporting to App


#define ENABLE_GxEPD2_GFX 0

#include <GxEPD2_BW.h>

#include "icons.h" // icon files in byte arrays
#include <Fonts/FreeMonoBold9pt7b.h>// load some fonts
#include <Fonts/FreeMonoBold12pt7b.h>
#include <Fonts/FreeMonoBold18pt7b.h>
int picWidth = 60;
int picHeight = 60;
long minOn = 1;// night mode run time in minutes
long minOff = 2;// night mode pause time in minutes
long nOffStart, nOnStart; // time that each part of night mode cycle begins
boolean nModeOn = false; // night mode ON cycle running
boolean nModeOff = false ;// night mode OFF cycle running
boolean acOn = false; // flag for if air conditioner is running or not

const unsigned char* icons[] =
{
  thermometer, humidity
};

float h, t; // temperature and humidity

GxEPD2_BW<GxEPD2_154_D67, GxEPD2_154_D67::HEIGHT> display(GxEPD2_154_D67(/*CS=5*/ SS, /*DC=*/ 22, /*RST=*/ 21, /*BUSY=*/ 4)); // GDEH0154D67
int xPos = 90;
int yPos = 85;

#include <IRremote.h>
// servo variables and libraries
#define COUNT_LOW 0
#define COUNT_HIGH 2222
#define TIMER_WIDTH 16
#include "esp32-hal-ledc.h"// for PWM management
//////////////
//#include <WiFi.h> //don't need for recent versions of Arduino IDE
#include <BlynkSimpleEsp32.h>
#include <DHT.h>

// You should get Auth Token in the Blynk App.
// Go to the Project Settings (nut icon).
char auth[] = "RdKpRobIPFL5sas9oYVksyDxXGbUCZCp";

// Your WiFi credentials.
// Set password to "" for open networks.
char ssid[] = "BELL014";
char pass[] = "4CCC3F2E";

#define DHTPIN 27 // temperature/humidity sensor pin was 4 - taken by display
int IR_RECEIVE_PIN = 34; // was 15 infrared receiver pin
int servoPin = 25; // servo signal pin
long elapsedOnTime, elapsedOffTime;
int counter = 0; // for display on APP to show that app is communicating with ESP32 - optional
IRrecv irrecv(IR_RECEIVE_PIN);// declare IR object
decode_results results;

// Uncomment whatever type you're using!
#define DHTTYPE DHT11     // DHT 11
//#define DHTTYPE DHT22   // DHT 22, AM2302, AM2321
//#define DHTTYPE DHT21   // DHT 21, AM2301

DHT dht(DHTPIN, DHTTYPE);// declare DHT sensor object
BlynkTimer timerBlynk;// variable 'timer' conflicts with IR library. Use different BlynkTimer object name

int pinStateAC;// value of virtual pin V0
int pinStateNM;// value of virtual pin V1
String tR, hR;// strings for sending temp and humidity to Blynk app


void setup()
{
  pinMode(IR_RECEIVE_PIN, INPUT_PULLUP);
  Serial.begin(115200);
  display.init(0);// 0 if don't want diagnostic messages. Otherwise 115200

  Blynk.begin(auth, ssid, pass);// open connection to Blynk over your home network

  dht.begin();
  irrecv.enableIRIn(); // Start the receiver
  // set up servo
  ledcSetup(1, 50, TIMER_WIDTH);
  ledcAttachPin(25, 1);
  //////
  // Setup a function to be called every second
  timerBlynk.setInterval(1000L, sendSensor);


  drawIcons();
  sendSensor();// populate temp and humidity display values to start
  Blynk.virtualWrite(V2, minOn);
  Blynk.virtualWrite(V3, minOff);
}

void loop()
{
  ////////// Night Mode Code //////////////
  if (pinStateNM == 1) {
   if(nModeOn==true) elapsedOnTime = millis() - nOnStart ;// calculate elapsed on-cycle time
    if(nModeOff == true) elapsedOffTime = millis()-nOffStart;// calculate elapsed on-cycle time
    if (nModeOn == true && abs(elapsedOnTime) > (minOn * 60000)) {

      // switch to off cycle
      acOn = false;
      cycleAC();
      nOffStart = millis();
      nModeOn = false;
      nModeOff = true;
    } else if (nModeOff == true && abs(elapsedOffTime) > (minOff * 60000)) {
     cycleAC();
      acOn = true;
      nOnStart = millis();
      nModeOn = true;
      nModeOff = false;
    }
  } else {
    
  }

  ////////// End Night Mode Code /////////////
  if (irrecv.decode(&results)) {
    String theVal = String(results.value);

    if (theVal.equals("4001918335")) {// CH+ AC On/Off
      if (pinStateAC == 0 && pinStateNM == 1) {
        pinStateAC = 1;
        stopNightMode();
        if (acOn == false) {
          acOn = true;
          cycleAC();
        }
        pinStateNM = 0;// turn off night mode of AC turned on manually
        Blynk.virtualWrite(V1, pinStateNM);
      } else if (pinStateAC == 1) {// ac IS ON
        acOn = false;
        pinStateAC = 0;
        cycleAC();
        Blynk.virtualWrite(V0, pinStateAC);// tell app that ac has been modified by IR remote
      } else if (pinStateAC == 1 && pinStateNM == 1 && acOn == true) {
        stopNightMode();
        pinStateNM = 0;
        Blynk.virtualWrite(V1, pinStateNM);
      } else if (pinStateAC == 0 && pinStateNM == 0) { // all off. turn ON ac
        acOn = true;
        cycleAC();
        pinStateAC = 1;
        Blynk.virtualWrite(V0, pinStateAC);
      }

    } else if (theVal.equals("3810010651")) {// CH- NightMode On/Off
      if (pinStateNM == 0 && pinStateAC == 0) { // all off turn on night mode
        startNightMode();
        pinStateNM = 1;
        acOn = true;
        cycleAC();
        Blynk.virtualWrite(V1, pinStateNM);
      } else if (pinStateNM == 1) { // if running, turn OFF night mode
        stopNightMode();
        pinStateNM = 0;
        if (acOn == true) { // if ac on, turn it off
          acOn = false;
          cycleAC();
        }
        Blynk.virtualWrite(V1, pinStateNM);
      } else if (pinStateNM == 0 and pinStateAC == 1) { // ac ON, move to night mode
        startNightMode();
        pinStateNM = 1;
        pinStateAC = 0;
        Blynk.virtualWrite(V1, pinStateNM);
        Blynk.virtualWrite(V0, pinStateAC);
      }

    } else if (theVal.equals("4034314555")) {//'-' NM On time increase
      if (minOn < 60) {
        minOn = minOn + 10;
      } else {
        minOn = 0;
      }
      showText();
      Blynk.virtualWrite(V2, minOn);
    } else if (theVal.equals("2747854299")) {
      if (minOff < 120) {
        minOff = minOff + 10;
      } else {
        minOff = 0;
      }
      showText();
      Blynk.virtualWrite(V3, minOff);
    }
    irrecv.resume(); // Receive the next value
  }


  Blynk.run();
  timerBlynk.run();// run a Blynk timer cycle
  delay(500);
}



void cycleAC() {// servo presses and releases power button
  // you may have to adjust these limits to suit your sweep angle
  for (int i = COUNT_LOW; i < COUNT_HIGH; i = i + 100) {
    ledcWrite(1, i);
    delay(20);
  }
  for (int i = COUNT_HIGH; i > COUNT_LOW; i = i - 100) {
    ledcWrite(1, i);
    delay(20);
  }
}

BLYNK_WRITE(V0) { // status change in AC On/Off pin on Blynk App
  pinStateAC = param.asInt();
  if (pinStateAC == 1 && pinStateNM == 1) {
    if (acOn == false) {// left night mode on a pause cycle
      Serial.println("         case 2");
      cycleAC();
      acOn = true;
    }
    Blynk.virtualWrite(V1, 0);
    pinStateNM = 0;
  } else if (pinStateAC == 1 && pinStateNM == 0) {
    cycleAC();
    acOn = true;
  } else if (pinStateNM == 0) {
    cycleAC();
    acOn = false;
  }
}

BLYNK_WRITE(V1) { // status change in Night Mode pin
  pinStateNM = param.asInt();
  if (pinStateNM == 1 && pinStateAC == 1) {
    if (acOn == false) {
      cycleAC();
      acOn = true;
    }
    startNightMode();
    Blynk.virtualWrite(V0, 0);
    pinStateAC = 0;
  } else if (pinStateNM == 1 && pinStateAC == 0) {
    cycleAC();
    acOn = true;
    startNightMode();
  } else if (pinStateAC == 0) {
    if (acOn == true) {
      cycleAC();
      acOn = false;
    }
    stopNightMode();
  }
}

BLYNK_WRITE(V2) { // get updated nightMode ON duration value
  minOn = param.asInt();
  minOn = (minOn / 10) * 10;
  if (minOn > 60) minOn = 60;
  Blynk.virtualWrite(V2, minOn);
}

BLYNK_WRITE(V3) { // get updated nightMode OFF duration value
  minOff = param.asInt();
  minOff = (minOff / 10) * 10;
  if (minOff > 120)minOff = 120;
  Blynk.virtualWrite(V3, minOff);
}

void drawIcons()
{ display.clearScreen();
  display.fillScreen(GxEPD_WHITE);

  // display.firstPage();

  do {

    display.setRotation(1);
    display.setFont(&FreeMonoBold9pt7b);
    display.setTextColor(GxEPD_BLACK);
    display.setCursor(5, 20);
    display.print("AC:");
    display.setCursor(5, 60);
    display.print("Night");
    display.setCursor(5, 80);
    display.print("Mode:");
    display.setCursor(5, 170);
    display.print("mON:");
    display.setCursor(5, 190);
    display.print("mOFF:");
  } while (display.nextPage());
  display.drawImage(icons[0], 140, 150, picWidth, picHeight, true, false, false);
  display.drawImage(icons[1], 60, 140, picWidth, picHeight, true, false, false);




  // //);

}

void showText()
{
  display.setPartialWindow(60, 0, 80, 200);
  display.firstPage();
  do
  {
    display.setFont(&FreeMonoBold12pt7b);
    display.setCursor(60, 170);
    display.print(minOn);
    display.setCursor(60, 190);
    display.print(minOff);
    display.setFont(&FreeMonoBold18pt7b);
    display.setCursor(80, 50);
    display.print(tR);
    display.setCursor(80, 130);
    display.print(hR);
    display.setCursor(60, 20);
    display.setFont(&FreeMonoBold12pt7b);
    if (pinStateAC == 1) {
      //display.setTextColor(GxEPD_WHITE);
      //display.fillRect(60,0,20,18,GxEPD_BLACK);
      display.print("ON");
    } else {
      //display.setTextColor(GxEPD_BLACK);
      //display.drawRect(60,0,20,18,GxEPD_WHITE);
      display.print("OFF");
    }
    display.setCursor(60, 80);
    if (pinStateNM == 1) {
      display.print("ON");
    } else {
      display.print("OFF");
    }


  }
  while (display.nextPage());
}


void startNightMode() {
  nOnStart = millis();
  nModeOn = true;
  nModeOff = false;
}

void stopNightMode() {
  nModeOn = false;
  nModeOff = false;
}

void sendSensor() {
  h = dht.readHumidity();
  t = dht.readTemperature();
  if (isnan(h) || isnan(t)) {
    Serial.println("Failed to read from DHT sensor!");
    return;
  }
  t = round(t * 10) / 10.0;// round to 1 deimal place
  tR = String(t).substring(0, 2); //cast to string to remove trailing 0 from float type
  //tR = tR + "\u00B0" + "C"; // add on degree symbol and C
  tR = tR + "C"; // add on degree symbol and C
  h = round(h * 10) / 10.0;
  //hR = String(h).substring(0, 4);
  hR = String(h).substring(0, 2);
  hR = hR + "%";
  Blynk.virtualWrite(V5, tR);// send temp and humidity values to app
  Blynk.virtualWrite(V6, hR);
  counter ++;
  if (counter > 200) counter = 0;
  Blynk.virtualWrite(V7, counter);// counter update is optional
  showText();// update display
}
